@@ -3,14 +3,12 @@ import gst
 import gobject
 import os
 import sys
-
-
+import thread
 
 import mutagen
 import mutagen.easyid3
 
 from gobject import GObject
-from threading import Thread
 from youamp import Song, known_exts, db_file
 from mutagen.id3 import ID3NoHeaderError
 
@@ -19,14 +17,13 @@ order["artist"] = "artist ASC, album ASC, tracknumber ASC"
 order["date"] = "date DESC, "+order["artist"]
 order["playcount"] = "playcount DESC, "+order["artist"]
 
-class Library(GObject, Thread):
-    __gsignals__ = {"update-complete": (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ())}
+class Library(GObject):
+    __gsignals__ = {"update-complete": (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (bool,))}
 
     def __init__(self):
-        Thread.__init__(self)
         GObject.__init__(self)
         
-        # set up connection for non thread communication
+        # set up connection
         self._conn = sqlite3.connect(db_file)
         self._cursor = self._conn.cursor()
 
@@ -103,13 +100,12 @@ class Library(GObject, Thread):
 
     # start update thread
     def update(self, folder):
-        self._music_dir = folder
-        self.start()
+        thread.start_new_thread(self._update, (folder,))
 
-    def _get_files(self):
+    def _get_files(self, folder):
         filelist = []
 
-        for root, dirs, files in os.walk(self._music_dir):
+        for root, dirs, files in os.walk(folder):
             for f in files:
                 if f.lower().endswith(known_exts):
                     uri = "file://"+os.path.join(root, f)
@@ -118,14 +114,16 @@ class Library(GObject, Thread):
 
         return filelist
  
-    def run(self):    
-        # local connection for Thread
+    def _update(self, folder):    
+        # local connection, so we can run as thread
         conn = sqlite3.connect(db_file)
         cursor = conn.cursor()
 
-        act_files = self._get_files()
+        act_files = self._get_files(folder)
         db_files = cursor.execute("SELECT uri FROM songs").fetchall()
-    
+        
+        updated = len(act_files) is not len(db_files)
+
         for uri, in db_files:
             try:
                 act_files.remove(str(uri))   # convert unicode
@@ -136,13 +134,8 @@ class Library(GObject, Thread):
         # add new files
         for uri in act_files:
             song = {"title": "None", "artist": "None", "album": "None", "genre": "None", "tracknumber": 0}
-            
-            # only look at local files
-            if not uri.startswith("file://"):
-                continue
-                
-            path = uri[7:] # strip prefix
-            path = path.replace("%25", "%") #unescape uri
+                            
+            path = gst.uri_get_location(uri)
             
             if path.lower().endswith("mp3"):
                 try:
@@ -192,7 +185,7 @@ class Library(GObject, Thread):
                 
         conn.commit()
         conn.close()
-        self.emit("update-complete")
+        self.emit("update-complete", updated)
 
 def check_db():
     conn = sqlite3.connect(db_file)
